import std/[json, httpclient, logging, options, sequtils, strutils]
import hff_notion_api_client
import hff_notion_api_client/config

import ./models, ./service

var notionClient = none[NotionClient]()

proc getNotionClient(cfg: HffEntryFormsApiConfig): NotionClient =
  if notionClient.isNone:
    notionClient = some(initNotionClient(NotionClientConfig(
      apiVersion: cfg.notionVersion,
      apiBaseUrl: cfg.notionApiBaseUrl,
      configDbId: "",
      integrationToken: cfg.integrationToken)))
  return notionClient.get

proc getEventProposalConfig*(cfg: HffEntryFormsApiConfig): EventProposalConfig =
  let notion = getNotionClient(cfg)

  var bodyJson: JsonNode
  try: bodyJson = notion.fetchDatabaseObject(cfg.eventParentId)
  except:
    raiseApiError(Http500,
      "unable to read event propsal configuration from notion API")

  let departmentOptionsJson = bodyJson{
    "properties", "Department", "multi_select", "options"}

  if departmentOptionsJson.isNil:
    raiseApiError(Http500,
      "missing read department values from Notion API-supplied event schema")

  return EventProposalConfig(
    departmentOptions: departmentOptionsJson.toSeq.mapIt(
      ( value: it["name"].getStr, color: it["color"].getStr )
    )
  )

proc createProposedEvent*(cfg: HffEntryFormsApiConfig, ep: EventProposal): bool =
  let notion = getNotionClient(cfg)

  try:
    discard notion.createDbPage(cfg.eventParentId, ep)
    return true
  except: return false
